window.Gleam.initSearch([{"doc":"qs","title":"qs","content":"# Gleam QS\n\nA query string parser for Gleam.\n\n## Install\n\n```\ngleam add valid\n```\n\n## Usage\n\nQS has two modules `qs` (Basic) and `qs_adv` (Advance).\n\n### Basic\n\nA query `(QueryBasic)` in basic is a `Dict(String, List(String))`.\n\nBasic has not concept of single or list values. Every value is a `List(String)`.\n\n### Advanced\n\nThe Advance module provides configuration to explicitly define how single and list values are serialized.\n\nA query `(QueryAdv)` is a `Dict(String, OneOrMany)`.\n\n### Basic parsing\n\n```gleam\nimport qs\nimport gleam/dict\n\n&quot;?color=red&amp;pet=cat&amp;pet=dog&quot;\n|&gt; qs.default_parse\n\n==\n\nOk(\n  dict.from_list(\n    [\n      #(&quot;color&quot;, [&quot;red&quot;]),\n      #(&quot;pet&quot;, [&quot;cat&quot;, &quot;dog&quot;]),\n    ]\n  )\n)\n```\n\n### Basic serialization\n\n```gleam\nimport qs\nimport gleam/dict\n\nlet query = [\n    #(&quot;color&quot;, [&quot;red&quot;]),\n    #(&quot;pet&quot;, [&quot;cat&quot;, &quot;dog&quot;])\n  ]\n  |&gt; dict.from_list\n\nqs.default_serialize(query)\n\n&gt; &quot;?color=red&amp;pet=cat&amp;pet=dog&quot;\n```\n\n### Advanced parsing\n\nBy default advanced uses a Rails like query scheme. E.g.\n\n```gleam\nimport qs_adv\nimport gleam/dict\n\n&quot;?color=red&amp;pets[]=cat&amp;pets[]=dog&quot;\n|&gt; qs_adv.default_parse\n\n==\n\nOk(\n  dict.from_list(\n    [\n      #(&quot;color&quot;, One(&quot;red&quot;)),\n      #(&quot;pets&quot;, Many([&quot;cat&quot;, &quot;dog&quot;])),\n    ]\n  )\n)\n```\n\nBut this is configurable.\n\n```gleam\nlet scheme = qs_adv.SchemeListAsSingleValue(\n  list_suffix: &quot;[]&quot;,\n  separator: &quot;|&quot;\n)\n\nlet config = qs_adv.default_config()\n  |&gt; qs_adv.with_scheme(scheme)\n\n&quot;?color=red&amp;pets[]=cat|dog&quot;\n  |&gt; qs_adv.parse(config)\n```\n\n### Advanced serialization\n\n```gleam\nimport qs_adv\nimport gleam/dict\n\nlet query = [\n    #(&quot;color&quot;, [&quot;red&quot;]),\n    #(&quot;pets&quot;, [&quot;cat&quot;, &quot;dog&quot;])\n  ]\n  |&gt; dict.from_list\n\nqs_adv.default_serialize(query)\n\n&gt; &quot;?color=red&amp;pets[]=cat&amp;pets[]=dog&quot;\n```\n","url":"index.html"},{"doc":"qs","title":"Config","content":"pub type Config {\n  Config(fail_on_invalid: Bool)\n}\n\nConfig(fail_on_invalid: Bool)\n\n\nSynonyms:\nqs.Config\nqs Config","url":"qs.html#Config"},{"doc":"qs","title":"Query","content":"pub type Query(v) =\n  Dict(String, v)\n\n\nSynonyms:\nqs.Query\nqs Query","url":"qs.html#Query"},{"doc":"qs","title":"QueryBasic","content":"pub type QueryBasic =\n  Query(List(String))\n\n\nSynonyms:\nqs.QueryBasic\nqs QueryBasic","url":"qs.html#QueryBasic"},{"doc":"qs","title":"default_config","content":"pub fn default_config() -&gt; Config\n\nSynonyms:\nqs.default_config\nqs default_config","url":"qs.html#default_config"},{"doc":"qs","title":"default_parse","content":"pub fn default_parse(\n  qs: String,\n) -&gt; Result(Dict(String, List(String)), String)\n Parse a query string.\n Values that cannot be parsed are ignored.\n\n ## Example\n\n \n &quot;?color=red&amp;tags=large&amp;tags=wool&quot;\n |&gt; qs.default_parse\n\n ==\n\n Ok(\n   dict.from_list(\n     [ #(&quot;color&quot;, [&quot;red&quot;]), #(&quot;tags&quot;, [&quot;large&quot;, &quot;wool&quot;]) ]\n   )\n )\n \n\nSynonyms:\nqs.default_parse\nqs default_parse","url":"qs.html#default_parse"},{"doc":"qs","title":"default_serialize","content":"pub fn default_serialize(\n  query: Dict(String, List(String)),\n) -&gt; String\n Serialize a query\n\n ## Example\n\n \n [ #(&quot;color&quot;, [&quot;red&quot;]), #(&quot;tag&quot;, [&quot;large&quot;, &quot;wool&quot;]) ]\n |&gt; qs.default_serialize\n\n ==\n\n &quot;?color=red&amp;tag=large&amp;tag=wool&quot;\n \nSynonyms:\nqs.default_serialize\nqs default_serialize","url":"qs.html#default_serialize"},{"doc":"qs","title":"delete","content":"pub fn delete(\n  query: Dict(String, a),\n  key: String,\n) -&gt; Dict(String, a)\n Delete a key from the query\nSynonyms:\nqs.delete\nqs delete","url":"qs.html#delete"},{"doc":"qs","title":"empty","content":"pub fn empty() -&gt; Dict(String, a)\n Make an empty Query\nSynonyms:\nqs.empty\nqs empty","url":"qs.html#empty"},{"doc":"qs","title":"get","content":"pub fn get(\n  query: Dict(String, a),\n  key: String,\n) -&gt; Result(a, String)\n Get values from the query\nSynonyms:\nqs.get\nqs get","url":"qs.html#get"},{"doc":"qs","title":"has_key","content":"pub fn has_key(query: Dict(String, a), key: String) -&gt; Bool\n Tell if the query has the given key\nSynonyms:\nqs.has_key\nqs has_key","url":"qs.html#has_key"},{"doc":"qs","title":"insert","content":"pub fn insert(\n  query: Dict(String, a),\n  key: String,\n  value: a,\n) -&gt; Dict(String, a)\n Insert a value in the query\n Replaces existing values\nSynonyms:\nqs.insert\nqs insert","url":"qs.html#insert"},{"doc":"qs","title":"merge","content":"pub fn merge(\n  a: Dict(String, a),\n  b: Dict(String, a),\n) -&gt; Dict(String, a)\n Merge two Querys.\n Second query takes precedence.\nSynonyms:\nqs.merge\nqs merge","url":"qs.html#merge"},{"doc":"qs","title":"parse","content":"pub fn parse(\n  input: String,\n  config: Config,\n) -&gt; Result(Dict(String, List(String)), String)\n\nSynonyms:\nqs.parse\nqs parse","url":"qs.html#parse"},{"doc":"qs","title":"with_fail_on_invalid","content":"pub fn with_fail_on_invalid(\n  config: Config,\n  value: Bool,\n) -&gt; Config\n\nSynonyms:\nqs.with_fail_on_invalid\nqs with_fail_on_invalid","url":"qs.html#with_fail_on_invalid"},{"doc":"qs","title":"qs","content":"","url":"qs.html"},{"doc":"qs_adv","title":"Config","content":"pub type Config {\n  Config(fail_on_invalid: Bool, scheme: Scheme)\n}\n\nConfig(fail_on_invalid: Bool, scheme: Scheme)\n\n\nSynonyms:\nqs_adv.Config\nqs_adv Config","url":"qs_adv.html#Config"},{"doc":"qs_adv","title":"OneOrMany","content":"pub type OneOrMany {\n  One(String)\n  Many(List(String))\n}\n\nOne(String)\n\n\nMany(List(String))\n\n\nSynonyms:\nqs_adv.OneOrMany\nqs_adv OneOrMany","url":"qs_adv.html#OneOrMany"},{"doc":"qs_adv","title":"QueryAdv","content":"pub type QueryAdv =\n  qs.Query(OneOrMany)\n\n\nSynonyms:\nqs_adv.QueryAdv\nqs_adv QueryAdv","url":"qs_adv.html#QueryAdv"},{"doc":"qs_adv","title":"Scheme","content":"pub type Scheme {\n  SchemeListAsMultipleValues(list_suffix: String)\n  SchemeListAsSingleValue(list_suffix: String, separator: String)\n}\n\nSchemeListAsMultipleValues(list_suffix: String)\n\n\nSchemeListAsSingleValue(list_suffix: String, separator: String)\n\n\nSynonyms:\nqs_adv.Scheme\nqs_adv Scheme","url":"qs_adv.html#Scheme"},{"doc":"qs_adv","title":"default_config","content":"pub fn default_config() -&gt; Config\n\nSynonyms:\nqs_adv.default_config\nqs_adv default_config","url":"qs_adv.html#default_config"},{"doc":"qs_adv","title":"default_parse","content":"pub fn default_parse(\n  qs: String,\n) -&gt; Result(Dict(String, OneOrMany), String)\n Parse a query string\n\n ## Example\n\n \n &quot;?color=red&amp;tags[]=large&amp;tags[]=wool&quot;\n |&gt; qs.default_parse\n\n ==\n\n Ok(\n   dict.from_list(\n     [ #(&quot;color&quot;, One(&quot;red&quot;)), #(&quot;tags&quot;, Many([&quot;large&quot;, &quot;wool&quot;])) ]\n   )\n )\n \n\nSynonyms:\nqs_adv.default_parse\nqs_adv default_parse","url":"qs_adv.html#default_parse"},{"doc":"qs_adv","title":"default_serialize","content":"pub fn default_serialize(\n  query: Dict(String, OneOrMany),\n) -&gt; String\n Serialize a query\n\n ## Example\n\n \n [ #(&quot;color&quot;, One(&quot;red&quot;)), #(&quot;tags&quot;, Many([&quot;large&quot;, &quot;wool&quot;])) ]\n |&gt; qs.serialize\n\n ==\n\n &quot;?color=red&amp;tags[]=large&amp;tags[]=wool&quot;\n \nSynonyms:\nqs_adv.default_serialize\nqs_adv default_serialize","url":"qs_adv.html#default_serialize"},{"doc":"qs_adv","title":"delete","content":"pub fn delete(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Dict(String, OneOrMany)\n\nSynonyms:\nqs_adv.delete\nqs_adv delete","url":"qs_adv.html#delete"},{"doc":"qs_adv","title":"empty","content":"pub fn empty() -&gt; Dict(String, OneOrMany)\n\nSynonyms:\nqs_adv.empty\nqs_adv empty","url":"qs_adv.html#empty"},{"doc":"qs_adv","title":"get","content":"pub fn get(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(OneOrMany, String)\n\nSynonyms:\nqs_adv.get\nqs_adv get","url":"qs_adv.html#get"},{"doc":"qs_adv","title":"get_as_bool","content":"pub fn get_as_bool(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(Bool, String)\n Attempt to get one value as a Bool.\n If the value is a list this fails.\n If the value cannot be parsed to a Bool this fails.\nSynonyms:\nqs_adv.get_as_bool\nqs_adv get_as_bool","url":"qs_adv.html#get_as_bool"},{"doc":"qs_adv","title":"get_as_float","content":"pub fn get_as_float(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(Float, String)\n Attempt to get one value as an Float.\n If the value is a list this fail.\nSynonyms:\nqs_adv.get_as_float\nqs_adv get_as_float","url":"qs_adv.html#get_as_float"},{"doc":"qs_adv","title":"get_as_int","content":"pub fn get_as_int(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(Int, String)\n Attempt to get one value as an Int.\n If the value is a list this fails.\n If the value cannot be parsed to an Int this fails.\nSynonyms:\nqs_adv.get_as_int\nqs_adv get_as_int","url":"qs_adv.html#get_as_int"},{"doc":"qs_adv","title":"get_as_list","content":"pub fn get_as_list(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; List(String)\n Get values from the query as a list of strings (regardless if one or many).\n If key is not present this defaults to an empty list.\nSynonyms:\nqs_adv.get_as_list\nqs_adv get_as_list","url":"qs_adv.html#get_as_list"},{"doc":"qs_adv","title":"get_as_list_of_bool","content":"pub fn get_as_list_of_bool(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(List(Bool), String)\n Attempt to get values as a list of Bool.\nSynonyms:\nqs_adv.get_as_list_of_bool\nqs_adv get_as_list_of_bool","url":"qs_adv.html#get_as_list_of_bool"},{"doc":"qs_adv","title":"get_as_list_of_float","content":"pub fn get_as_list_of_float(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(List(Float), String)\n Attempt to get values as a list of Float.\nSynonyms:\nqs_adv.get_as_list_of_float\nqs_adv get_as_list_of_float","url":"qs_adv.html#get_as_list_of_float"},{"doc":"qs_adv","title":"get_as_list_of_int","content":"pub fn get_as_list_of_int(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(List(Int), String)\n Attempt to get values as a list of Int.\nSynonyms:\nqs_adv.get_as_list_of_int\nqs_adv get_as_list_of_int","url":"qs_adv.html#get_as_list_of_int"},{"doc":"qs_adv","title":"get_as_string","content":"pub fn get_as_string(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(String, String)\n Attempt to get one value as a String.\n If the value is a list this fails.\nSynonyms:\nqs_adv.get_as_string\nqs_adv get_as_string","url":"qs_adv.html#get_as_string"},{"doc":"qs_adv","title":"has_key","content":"pub fn has_key(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Bool\n Tell if the query has the given key.\nSynonyms:\nqs_adv.has_key\nqs_adv has_key","url":"qs_adv.html#has_key"},{"doc":"qs_adv","title":"insert","content":"pub fn insert(\n  query: Dict(String, OneOrMany),\n  key: String,\n  value: OneOrMany,\n) -&gt; Dict(String, OneOrMany)\n Insert value.\nSynonyms:\nqs_adv.insert\nqs_adv insert","url":"qs_adv.html#insert"},{"doc":"qs_adv","title":"insert_list","content":"pub fn insert_list(\n  query: Dict(String, OneOrMany),\n  key: String,\n  values: List(String),\n) -&gt; Dict(String, OneOrMany)\n Set a list of values in the query.\nSynonyms:\nqs_adv.insert_list\nqs_adv insert_list","url":"qs_adv.html#insert_list"},{"doc":"qs_adv","title":"insert_one","content":"pub fn insert_one(\n  query: Dict(String, OneOrMany),\n  key: String,\n  value: String,\n) -&gt; Dict(String, OneOrMany)\n Set a unique value in the query.\nSynonyms:\nqs_adv.insert_one\nqs_adv insert_one","url":"qs_adv.html#insert_one"},{"doc":"qs_adv","title":"maybe_get_as_list","content":"pub fn maybe_get_as_list(\n  query: Dict(String, OneOrMany),\n  key: String,\n) -&gt; Result(List(String), String)\n Get values from the query as a list of strings.\n If key is not present this returns an Error.\nSynonyms:\nqs_adv.maybe_get_as_list\nqs_adv maybe_get_as_list","url":"qs_adv.html#maybe_get_as_list"},{"doc":"qs_adv","title":"merge","content":"pub fn merge(\n  a: Dict(String, OneOrMany),\n  b: Dict(String, OneOrMany),\n) -&gt; Dict(String, OneOrMany)\n\nSynonyms:\nqs_adv.merge\nqs_adv merge","url":"qs_adv.html#merge"},{"doc":"qs_adv","title":"parse","content":"pub fn parse(\n  input: String,\n  config: Config,\n) -&gt; Result(Dict(String, OneOrMany), String)\n\nSynonyms:\nqs_adv.parse\nqs_adv parse","url":"qs_adv.html#parse"},{"doc":"qs_adv","title":"scheme_rails_like","content":"pub fn scheme_rails_like() -&gt; Scheme\n\nSynonyms:\nqs_adv.scheme_rails_like\nqs_adv scheme_rails_like","url":"qs_adv.html#scheme_rails_like"},{"doc":"qs_adv","title":"serialize","content":"pub fn serialize(\n  input: Dict(String, OneOrMany),\n  config: Config,\n) -&gt; String\n\nSynonyms:\nqs_adv.serialize\nqs_adv serialize","url":"qs_adv.html#serialize"},{"doc":"qs_adv","title":"with_scheme","content":"pub fn with_scheme(config: Config, scheme: Scheme) -&gt; Config\n\nSynonyms:\nqs_adv.with_scheme\nqs_adv with_scheme","url":"qs_adv.html#with_scheme"},{"doc":"qs_adv","title":"qs_adv","content":"","url":"qs_adv.html"}]);